"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const fs = require("fs");
const path = require("path");
const rimraf = require("rimraf");
const checksum = require("checksum");
const decompress = require('decompress');
const decompressUnzip = require('decompress-unzip');
const targz = require('tar.gz');
const tar = require('tar');
const zlib = require('zlib');
class Archiver {
    // leaving this option around as Git LFS has a whole bunch of variation
    // with how it's archived that we need to workaround here
    static extractAndFlatten(source, destination, extension) {
        let options = {
            // strip any leading directory information
            strip: 1,
            // only extract a given set of file extensions
            filter: (file) => path.extname(file.path) === extension
        };
        return decompress(source, destination, options);
    }
    // because Git packages may contain symlinks, we're gonna use some more
    // low-level libraries to ensure we preserve them when unpacking
    static extractGzip(source, destination) {
        return new Promise((resolve, reject) => {
            const extractor = tar.Extract({ path: destination })
                .on('error', (error) => reject(error))
                .on('end', () => resolve());
            fs.createReadStream(source)
                .on('error', (error) => reject(error))
                .pipe(zlib.Gunzip())
                .pipe(extractor);
        });
    }
    // TODO: once all upstream releases are done with `tgz` this code will not be necessary
    static extractZip(source, destination) {
        let options = {
            plugins: [
                decompressUnzip()
            ]
        };
        return decompress(source, destination, options);
    }
    static unpackGitLFS(platform, source, destination) {
        if (platform === 'win32') {
            const nestedPath = path.join(destination, 'mingw64', 'libexec', 'git-core');
            return Archiver.extractAndFlatten(source, nestedPath, '.exe');
        }
        else if (platform === 'darwin' || platform === 'linux') {
            const nestedPath = path.join(destination, 'libexec', 'git-core');
            return Archiver.extractAndFlatten(source, nestedPath, '');
        }
        else {
            return Promise.reject(`unable to unpack Git LFS as platform '${platform}' is not currently supported`);
        }
    }
    static unpackGit(source, directory) {
        if (fs.existsSync(directory)) {
            rimraf.sync(directory);
        }
        if (path.extname(source) === '.zip') {
            return Archiver.extractZip(source, directory);
        }
        else {
            return Archiver.extractGzip(source, directory);
        }
    }
    static unpackAll(platform, config, temporaryDirectory) {
        return __awaiter(this, void 0, void 0, function* () {
            const temporaryGitDownload = path.join(temporaryDirectory, config.git.fileName);
            const temporaryGitDirectory = path.join(temporaryDirectory, platform, config.git.version);
            const temporaryGitLFSDownload = path.join(temporaryDirectory, config.lfs.fileName);
            yield Archiver.unpackGit(temporaryGitDownload, temporaryGitDirectory);
            yield Archiver.unpackGitLFS(platform, temporaryGitLFSDownload, temporaryGitDirectory);
        });
    }
    static create(directory, file) {
        const read = targz().createReadStream(directory);
        const write = fs.createWriteStream(file);
        read.pipe(write);
        return Promise.resolve();
    }
    static output(file) {
        return new Promise((resolve, reject) => {
            checksum.file(file, { algorithm: 'sha256' }, (err, hash) => {
                if (err) {
                    reject(err);
                }
                console.log(`File: ${file}`);
                console.log(`SHA256: ${hash}`);
                resolve();
            });
        });
    }
}
exports.Archiver = Archiver;
//# sourceMappingURL=archiver.js.map