"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const request = require("request");
const ProgressBar = require("progress");
const fs = require("fs");
const path = require("path");
const checksum = require("checksum");
const rimraf = require("rimraf");
const verifyFile = (file, expected) => {
    return new Promise((resolve, reject) => {
        checksum.file(file, { algorithm: 'sha256' }, (_, hash) => {
            if (hash !== expected) {
                console.log(`checksum failed: got '${hash}' but expected '${expected}'`);
            }
            resolve(hash === expected);
        });
    });
};
class Downloader {
    static getReleaseAssets(owner, repo, tag) {
        const url = `https://api.github.com/repos/${owner}/${repo}/releases/tags/${tag}`;
        // TODO: look for environment variable for a GitHub token
        const options = {
            url: url,
            headers: {
                'User-Agent': 'git-kitchen-sink'
            }
        };
        return new Promise((resolve, reject) => {
            request.get(options, (error, response, body) => {
                if (error) {
                    reject(error);
                }
                if (response.statusCode === 403) {
                    reject('Oops, it seems you\'ve been rate-limited on this IP address. Set GITHUB_API_TOKEN to an empty personal acceess token from https://github.com/settings/tokens to workaround this');
                }
                if (response.statusCode !== 200) {
                    reject('unexpected response code: ' + response.statusCode);
                }
                const json = JSON.parse(body);
                resolve(json.assets);
            });
        });
    }
    static resolveGitLFSForPlatform(version, platform) {
        return __awaiter(this, void 0, void 0, function* () {
            const assets = yield Downloader.getReleaseAssets('github', 'git-lfs', `v${version}`);
            const label = platform;
            const asset = assets.find(a => a.label === label);
            if (!asset) {
                return Promise.reject(`unable to find asset on Git-LFS release: ${label}`);
            }
            return Promise.resolve({ url: asset.browser_download_url, fileName: asset.name });
        });
    }
    static download(alias, url, destination) {
        return new Promise((resolve, reject) => {
            console.log(`Downloading ${url}`);
            const options = {
                url: url,
                headers: {
                    'User-Agent': 'git-kitchen-sink'
                },
                encoding: null
            };
            const req = request.get(options, (error, response, body) => {
                // console.log(`Writing contents to ${url} to ${destination}`)
                const callback = (error) => {
                    // TODO: i'm not sure if this is even valid
                    if (error) {
                        reject(error);
                    }
                    if (response.statusCode !== 200) {
                        reject(`unexpected response code: ${response.statusCode}`);
                    }
                    resolve();
                };
                fs.createWriteStream(destination).write(body, callback);
            });
            req.on('response', function (res) {
                const len = parseInt(res.headers['content-length'], 10);
                console.log();
                const bar = new ProgressBar('Downloading ' + alias + ' [:bar] :percent :etas', {
                    complete: '=',
                    incomplete: ' ',
                    width: 50,
                    total: len
                });
                res.on('data', (chunk) => {
                    bar.tick(chunk.length);
                });
                res.on('end', function () {
                    console.log('\n');
                });
            });
        });
    }
    static downloadGit(config, temporaryDirectory) {
        return __awaiter(this, void 0, void 0, function* () {
            const destination = path.join(temporaryDirectory, config.git.fileName);
            if (fs.existsSync(destination)) {
                const valid = yield verifyFile(destination, config.git.checksum);
                if (valid) {
                    return Promise.resolve();
                }
                else {
                    console.log('cached Git found but doesn\'t match checksum, trying again...');
                    rimraf.sync(destination);
                    return Downloader.download('Git', config.git.source, destination);
                }
            }
            else {
                yield Downloader.download('Git', config.git.source, destination);
                const isValid = yield verifyFile(destination, config.git.checksum);
                if (!isValid) {
                    rimraf.sync(destination);
                    return Promise.reject('downloaded Git doesn\'t match checksum, aborting...');
                }
            }
        });
    }
    static downloadGitLFS(config, temporaryDirectory) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = config.lfs.url;
            const destination = path.join(temporaryDirectory, config.lfs.fileName);
            if (fs.existsSync(destination)) {
                const valid = yield verifyFile(destination, config.lfs.checksum);
                if (valid) {
                    return Promise.resolve();
                }
                else {
                    console.log('cached Git LFS found but doesn\'t match checksum, trying again...');
                    rimraf.sync(destination);
                    return Downloader.download('Git LFS', url, destination);
                }
            }
            else {
                yield Downloader.download('Git LFS', url, destination);
                const valid = yield verifyFile(destination, config.lfs.checksum);
                if (valid) {
                    return Promise.resolve();
                }
                else {
                    rimraf.sync(destination);
                    return Promise.reject('downloaded Git LFS doesn\'t match checksum, aborting...');
                }
            }
        });
    }
}
exports.Downloader = Downloader;
//# sourceMappingURL=downloader.js.map